/* 
 * Copyright (c) 2000-2003, Andy Gilligan
 * All rights reserved.
 * 
 * This software is not subject to any export provision of the United
 * States Department of Commerce, and may be exported to any country
 * or planet.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice immediately at the beginning of the file, this list of
 *    conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* $Id: mempool.c,v 1.4 2003/04/29 04:49:05 ajg Exp $ */

#include <stdlib.h>
#include "mempool.h"

void *palloc(struct list *list, size_t sz)
{
	struct mnode *node = 0;

	if (list) {
		if ((node = calloc(1, sizeof(struct mnode)))) {
			if ((node->ptr = calloc(1, sz))) {
				if (list->head == 0) {
					list->head = node;
				} else {
					node->prev = list->tail;
					node->id = node->prev->id + 1;
					list->tail->next = node;
				}

				list->tail = node;
			} else {
				free(node);
				node = 0;
			}
		}
	}
	
	if (node)
		return node->ptr;
	else
		return 0;
}

void pfree(struct list *list)
{
	struct mnode *node;

	if (list) {
		while ((node = list->head)) {
			if (node->ptr) {
				free(node->ptr);
				node->ptr = 0;
			}
		
			if (node->next)
				node->next->prev = 0;
		
			list->head = node->next;
			free(node);
			node = 0;
		}
	}
}

