/*
 * Copyright (c) 2000-2003, Andy Gilligan
 * All rights reserved.
 * 
 * This software is not subject to any export provision of the United
 * States Department of Commerce, and may be exported to any country
 * or planet.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice immediately at the beginning of the file, this list of
 *    conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */ 

/* $Id: parse.c,v 1.4 2003/04/29 04:49:05 ajg Exp $ */

#include <string.h>

// quick hack, but we don't need it to do anything important :)
#define ISWS(x) ((x) <= ' ')

// chop up a string into words, inserting null's in place of whitespace
// and inserting a pointer to the word in our 'opt' array.
void parse(char **opt, char *line, int maxent)
{
	int c, x;
	size_t len;

	// just be sure we have a buffer
	if (line == 0)
		return;
	
	// we test this separately, as some buggy compilers
	// will test both conditions when 'if (line == 0 || line[0] == 0)'
	// this will segfault... which is bad :)
	if (*line == '\0')
		return;
	
	// get string length, and set our first element to be
	// the start of the string.
	len = strlen(line);
	opt[0] = line;

	// while current position is less than length, and we haven't
	// added more than 'maxent' entries...
	for (c = x = 1; (c < (int) len) && (x < (maxent - 1)); c++) {
		if (ISWS(line[c])) {
			line[c] = '\0';
			continue;
		}

		// current char isn't null, but is the previous null?
		// if it is, we've got our next entry.
		if (line[c - 1] == '\0')
			opt[x++] = &line[c];
	}

	// terminate our array
	opt[x] = 0;
	return;
}

