/* 
 * Copyright (c) 2000-2003, Andy Gilligan
 * All rights reserved.
 * 
 * This software is not subject to any export provision of the United
 * States Department of Commerce, and may be exported to any country
 * or planet.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice immediately at the beginning of the file, this list of
 *    conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */ 

/* $Id: sws.c,v 1.5.2.2 2003/06/11 03:26:56 ajg Exp $ */

#include <string.h>

// strips leading & trailing whitespace from a string
// shifts chars down to fill leading whitespace.

char *sws(char *str)
{
	int c;
	size_t len;
	char *ptr;

	if (str == 0)
		return str;

	// skip leading whitespace
	for (ptr = str; (*ptr != '\0') && (*ptr <= ' '); ptr++)
		;

	// do we have any string left?
	if ((len = strlen(ptr))) {

		// skip trailing whitespace, running backwards
		for (c = len - 1; c; c--) {
			// if we have a non-whitespace char, we're done
			if (ptr[c] > ' ') {
				ptr[c + 1] = '\0';
				break;
			}
		}
	}

	// shift usable chars down
	for (c = 0; len && (c < (int) len); c++)
		str[c] = ptr[c];
	
	str[c] = '\0';
	return str;
}

